# -*- coding: utf-8 -*-
#
# MONK Automated Testing Framework
#
# Copyright (C) 2012-2013 DResearch Fahrzeugelektronik GmbH, project-monk@dresearch-fe.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 3 of the License, or (at you option) any later version.
#

from nose import tools as nt

from monk_tf import ssh_conn as msc

def test_simple():
    """check if creating a SshConn object works
    """
    # prepare
    test_host = None
    # execute
    sut = msc.SshConn(test_host)
    # assert
    nt.ok_(sut, "should contain a monk_tf.ssh_conn.SshConn object, but instead contains this: '{}'".format(sut))


def test_cmd_closes_without_exception():
    """check if closing without exception is called without waiting
    """
    # prepare
    sut = MockSshConn()
    expected = ['close()']
    # execute
    sut.cmd()
    # assert
    nt.eq_(expected, sut.calls, "expected '{}', but got '{}'".format(expected, sut.calls))


def test_cmd_closes_with_exception():
    """check if closing with exception is called without waiting
    """
    # prepare
    sut = MockSshConn(close_callback=raise_exception)
    expected = ['close()', 'wait_closed()']
    # execute
    sut.cmd()
    # assert
    nt.eq_(expected, sut.calls, "expected '{}', but got '{}'".format(expected, sut.calls))


# --------------- HELPERS -------------------------------

def raise_exception():
    """ a callback that raises a simple Exception
    """
    raise Exception()


class MockSshConn(msc.SshConn):


    def __init__(self, close_callback=None):
        mock_host = "ignore me"
        super(MockSshConn, self).__init__(mock_host)
        self.calls = []
        if close_callback is not None:
            self.close_callback = close_callback
        else:
            self.close_callback = self.empty_callback


    def cmd(self, txt=None):
        exec_txt = txt if txt is not None else "ignore the command text"
        return super(MockSshConn, self).cmd(exec_txt)


    def empty_callback(self):
        pass


    def _session(self):
        return self


    def open_session(self):
        return self


    def execute(self, throw_away):
        return 0


    def read(self, throw_away):
        return ""


    def close(self):
        self.calls.append('close()')
        self.close_callback()
        return 0


    def wait_closed(self):
        self.calls.append('wait_closed()')


    def exit_status(self):
        return 0



# -*- coding: utf-8 -*-
#
# MONK Automated Testing Framework
#
# Copyright (C) 2012-2013 DResearch Fahrzeugelektronik GmbH, project-monk@dresearch-fe.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 3 of the License, or (at you option) any later version.
#

from nose import tools as nt

from monk_tf import serial_io as sio


def test_simple():
    """ serial_io: create a SerialIO Object without problems
    """
    # set up
    # exec
    sut = sio.SerialIO()
    # assert
    nt.ok_(isinstance(sut,sio.SerialIO),
        "should have type SerialIO, but has {}".format(type(sut)))


def test_cmd_hello_world():
    """ serial_io: cmd("hello") returns "world"
    """
    # set up
    any_txt = "hello"
    expected_out = "world"
    expected_calls = ["write('{}\n')".format(any_txt), "readall()"]
    sut = MockSerial()
    sut.out = "hello\n{}\n<prompt>$ ".format(expected_out)
    # exec
    out = sut.cmd(any_txt)
    # assert
    nt.eq_(expected_out, out)
    nt.eq_(expected_calls, sut.calls)


def test_cmd_with_windows_newlines():
    """ serial_io: make sure that windows newlines get replaced
    """
    # set up
    any_txt = "hello"
    expected_out = "world"
    expected_calls = ["write('{}\n')".format(any_txt), "readall()"]
    sut = MockSerial()
    sut._linesep = "\n"
    sut.out = "hello\n\r{}\n\r<prompt>$ ".format(expected_out)
    # exec
    out = sut.cmd(any_txt)
    # assert
    nt.eq_(expected_out, out)
    nt.eq_(expected_calls, sut.calls)


class MockSerial(sio.SerialIO):


    def __init__(self):
        self.calls = []


    def readall(self):
        self.calls.append("readall()")
        return self.out

    def write(self,msg):
        self.calls.append("write('{}')".format(msg))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2012, 2013 DResearch Fahrzeugelektronik GmbH
# Maintained by project-monk@dresearch-fe.de
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#



# MONK automated integration test framework public API

from devicetestcase import DeviceTestCase
from device         import Device
from connection     import Connection
from ssh_conn       import SshConn
from serial_conn    import SerialConn
from bcc            import Bcc



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# MONK Automated Testing Framework
#
# Copyright (C) 2013 DResearch Fahrzeugelektronik GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#

from setuptools import setup

project = "monk_tf"
version = "0.1.2"

setup(
    name=project,
    version=version,
    description = "a test framework for embedded systems",
    author = "DResearch Fahrzeugelektronik GmbH",
    author_email = "project-monk@dresearch-fe.de",
    url="https://github.com/DFE/MONK",
    packages=[project],
    install_requires = [
        "pyserial >=2.5",
        "pylibssh2 >=1.0.1"
    ],provides = [
        "{} ({})".format(project, version)
    ],
    test_suite = "nose.collector"
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012, 2013 DResearch Fahrzeugelektronik GmbH
# Maintained by project-monk@dresearch-fe.de
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from nose import tools as nt

from monk_tf import serial_conn as msc


def test_simple():
    """checks if creating a SerialConn object works
    """
    #nothing to prepare
    #
    #execute
    sut = msc.SerialConn()
    #assert
    nt.ok_(sut, "should contain a monk_tf.serial_conn.SerialConn object, but instead contains this: '{}'".format(sut))


def test_setting_read_timeout():
    """shall test if read timeouts are set properly

    The test mocks the pyserial API for setting this timeout. The API
    contains a class Serial with the parameter `timeout`
    (http://pyserial.sourceforge.net/pyserial_api.html#serial.Serial.timeout).
    This is originally defined in `pyserial/serial/util.py` in class
    `SerialBase` with the property-function `setTimeout`.

    This test comes wit the MockSerial class underneath the TestCase, which
    also contains this setter. But instead of setting real timeouts it
    simply stores all values in a list this test. Afterwards those values
    can be evaluated to find if the API seriously was used as expected.
    """
    #prepare
    start_rto = 591
    set_rto = -234
    #this contains always 2 calls, because read_until always writes both
    # read- and writeTimeout
    expected_rto = [
            set_rto,   #written into old_rto
            set_rto,   #written into old_wto
            start_rto, #written from old_rto
            None       #written from old_wto, which wasn't set
    ]
    target = "doesn't matter,rto" #just needed to fulfill API
    sut = MockSerial(rto=start_rto, target=target)
    #execute
    msc.SerialConn.read_until(sut, target, timeout=set_rto)
    #assert
    nt.eq_(expected_rto, sut.results,
            "expected: '{}', found: '{}'".format(expected_rto, sut.results))


def test_setting_write_timeout():
    """shall test if write timeouts are set properly

    The test mocks the pyserial API for setting this timeout. The API
    contains a class Serial with the parameter `writeTimeout`
    (http://pyserial.sourceforge.net/pyserial_api.html#serial.Serial.writeTimeout).
    This is originally defined in `pyserial/serial/util.py` in class
    `SerialBase` with the property-function `setTimeout`.

    This test comes wit the MockSerial class underneath the TestCase, which
    also contains this setter. But instead of setting real timeouts it
    simply stores all values in a list this test. Afterwards those values
    can be evaluated to find if the API seriously was used as expected.
    """
    #prepare
    start_wto = 195
    set_wto = -342
    #this contains always 2 calls, because read_until always writes both
    # read- and writeTimeout
    expected_wto = [
            set_wto,   #written into old_rto
            set_wto,   #written into old_wto
            None,      #written from old_rto, which wasn't set
            start_wto  #written from old_wto
    ]
    target = "doesn't matter,wto" #just needed to fulfill API
    sut = MockSerial(wto=start_wto, target=target)
    #execute
    msc.SerialConn.read_until(sut, target, timeout=set_wto)
    #assert
    nt.eq_(expected_wto, sut.results,
            "expected: '{}', found: '{}'".format(expected_wto, sut.results))


class MockSerial(msc.SerialConn):
    """mocks the :py:class:`serial.Serial` API

    It provides API like :py:class:`serial.Serial` and stores setter call
    results for comparison.
    """

    def __init__(self,rto=None, wto=None,target=None):
        self._rto = rto
        self._wto = wto
        self.target = target
        self.results = []
        self._logger = logging.getLogger(__name__)


    @property
    def timeout(self):
        return self._rto


    @timeout.setter
    def timeout(self,rto):
        self.results.append(rto)
        self._rto = rto


    @property
    def writeTimeout(self):
        return self._wto


    @writeTimeout.setter
    def writeTimeout(self,wto):
        self.results.append(wto)
        self._wto = wto

    def read(self):
        """ignore me.

        :py:func:`serial_conn.SerialConn.read_until()` expects to call this
        function, which is the only reason for it's existance.
        """
        return self.target



# -*- coding: utf-8 -*-
#
# MONK automated test framework
#
# Copyright (C) 2013 DResearch Fahrzeugelektronik GmbH
# Written and maintained by MONK Developers <project-monk@dresearch-fe.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 3 of the License, or (at your option) any later version.
#

import os
import time
import logging

import serial

class SerialIO(serial.Serial):
    """ console like interface on top of pyserial

    The job of this class is to allow a console like input-output behaviour
    between the user of this class and a serial interface. To achieve this some
    abstraction layers are necessary.
    """


    def __init__(self, *args, **kwargs):
        """
        :param prompt: set a prompt for the communication
        """
        print "wrong init :("
        self._logger = logging.getLogger(__name__)
        if "linesep" in args:
            self.linesep = args["linesep"]
            args.pop("linesep")
        super(SerialIO,self).__init__(*args, **kwargs)


    def cmd(self, msg):
        """ send a command and receive it's output

        :param msg: the command that shall be sent, with or without line
                    separator in the end
        :return: the output of the command as created by target device
        """
        # a command will only be executed, if it ends in a linebreak
        self.write(msg.strip() + self.linesep)
        # remove first line (the cmd itself), last line (the next prompt)
        # and unnecesary \r characters from the output
        return self.linesep.join(self.readall().replace("\r","").split("\n")[1:-1])

    @property
    def linesep(self):
        """ In most cases ``os.linesep``.
        """
        try:
            return self._linesep
        except AttributeError:
            self.linesep = os.linesep
            return self._linesep

    @linesep.setter
    def linesep(self, lsep):
        self._linesep = lsep
